define(["jquery", "underscore", "backbone", "tpls", "context", "grid", "models/table/schema"],
    function ($, _, Backbone, Templates, Context, Grid, Schema) {
        return Backbone.View.extend({
            template: Templates["builder/structure"],
            gridSettings: {
                tableClass: "table table-bordered table-condensed",
                sortable: false,
                emptyRow: true
            },
            columns: ["Name", "Columns", "Type", "Sub_Part", "Non_Unique"],
            gridColumns: function () {
                this.charsets.unshift("");
                var charsets = {values: this.charsets};
                var nulls = {values: ["YES", "NO"]};
                return _.map(this.columns, function (col) {
                    if (col === "CharacterSet") {
                        return {name: col, type: "string", editor: "SelectEditor", editorProps: charsets};
                    } else if (col === "Null") {
                        return {name: col, type: "string", editor: "SelectEditor", editorProps: nulls};
                    } else {
                        return {name: col, type: "string", editor: "BasicEditor"};
                    }
                });
            },
            initialize: function () {
                _.bindAll(this, "updateColumn", "createColumn", "renderGrid", "error", "changeError", "changeSuccess");
            },
            render: function () {
                this.$el.html(this.template());

                var schema = new Schema();
                schema.fetch({success: this.renderGrid, error: this.error});

                return this;
            },
            /**
             * Handle schema fetch errors
             * @param model
             * @param error
             */
            error: function (model, error) {
                alert("Failed to fetch table schema.")
            },
            /**
             * Render sensei grid
             * @param model
             * @param data
             */
            renderGrid: function (model, data) {

                // store initial data for later use
                this.initialData = data["indexes"];
                this.charsets = data["charsets"];

                // get sensei grid container element
                var $el = this.$(".sensei-grid-builder");

                // grid initialization
                this.grid = $el.grid(
                    this.initialData,
                    this.gridColumns(),
                    this.gridSettings
                );

                // register basic editor
                this.grid.registerEditor(BasicEditor);
                this.grid.registerEditor(SelectEditor);

                // listen to events
                this.grid.events.on("editor:save", this.updateColumn);
                this.grid.events.on("row:save", this.createColumn);

                // render grid
                this.grid.render();
            },
            /**
             * Update an existing column
             * @param data
             * @param $cell
             */
            updateColumn: function (data, $cell) {

                if (!this.grid.getCellStatus($cell)) {
                    console.log("nope updateColumn");
                    return;
                }

                console.log("updateColumn", data, $cell);
                _.forIn(data, function (value, param) {
                    var $row = this.grid.getCellRow($cell);
                    var index = $row.index();

                    if (this.initialData[index] && this.initialData[index]["Field"]) {

                        var column = this.initialData[index]["Field"];
                        var data = {param: param, value: value, column: column};
                        var schema = new Schema();

                        schema.save(data, {success: this.changeSuccess, error: this.changeError});
                    } else {
                        this.changeError();
                    }
                }, this);
            },
            /**
             * Create a new column
             * @param data
             * @param $row
             */
            createColumn: function (data, $row) {
                console.log("create column", data, $row);
                // a little workaround for backbone
                var schema = new Schema();
                schema.setCreateURL();
                var grid = this.grid;
                schema.save(data, {success: function (model, response, options) {
                    grid.setRowSaved($row);
                }, error: this.changeError});
            },
            changeError: function (model, response, options) {
                if (response["responseJSON"] && response["responseJSON"]["error"]) {
                    console.log(response);
                    alert("Schema update failed. " + response["responseJSON"]["error"]);
                } else {
                    alert("Schema update failed: unexpected error.");
                }
            },
            changeSuccess: function (model, response, options) {
                // do nothing for now
            }
        });
    });
