define(['jquery', 'underscore', 'backbone', 'tpls', 'models/connection', 'events', 'context'],
    function ($, _, Backbone, Templates, Connection, Events, Context) {
      return Backbone.View.extend({
        template: Templates["editconnection"],
        showConnectionError: false,
        modal: false, // if set true view will adjusted for modal embed
        defaultPorts: {
          "mysql": 3306,
          "postgresql": 5432,
          "redshift": 5439
        },
        events: {
          "submit form": "save",
          "click .connect-btn": function () {
            $(this).parents("form").submit();
          },
          "change #field_type": "setDefaultPort"
        },
        save: function (e) {
          e.preventDefault();

          this.$("p.error").remove();

          var data = Context.serializeObj($(e.target));
          var connection = this.model ? this.model : new Connection();

          // quick workaround when ssl value is missing, we should force the default value
          if (!data.ssl) {
            data.ssl = false;
          }

          connection.save(data, {
            success: function (model) {
              Context.connection = model.id;
              Context.driver = model.get("driver");
              Events.trigger('connection:create', model);
              Events.trigger('connection:change', model);
            },
            error: function (model, xhr) {

              var res = null;

              try {
                res = $.parseJSON(xhr.responseText);
              } catch (e) {
                alert("Unexpected error.");
              }

              if (_.isObject(res.errors)) {
                _.each(res.errors, function (error, key) {
                  var $error = $("<p>").addClass("error help-block").text(error);
                  var $field = this.$("input").filter(function () {
                    return $(this).attr("name") === key;
                  });
                  $error.insertAfter($field);
                }, this);
              }
            }
          });
        },
        setDefaultPort: function () {
          var driver = this.$("#field_type").val();
          var $portField = this.$("input[name=port]");
          var currentPort = parseInt($portField.val());
          var ports = _.values(this.defaultPorts);
          if (($portField.val().length === 0 || _.indexOf(ports, currentPort) > -1)
              && this.defaultPorts[driver]) {
            $portField.val(this.defaultPorts[driver]);
          }
        },
        initialize: function (opts) {
          if (opts && opts.model) {
            this.model = opts.model;
          }
          if (opts && opts.showConnectionError) {
            this.showConnectionError = opts.showConnectionError;
          }
          if (opts && opts.modal) {
            this.modal = opts.modal;
          }
        },
        render: function () {
          var data = this.model ? this.model.toJSON() : {};
          data.showConnectionError = this.showConnectionError;
          data.modal = this.modal;
          this.$el.html(this.template(data));
          this.setDefaultPort();
          return this;
        }
      });
    });
