define(["jquery", "underscore", "backbone", "tpls", "events", "models/user", "context", "models/license",
      "collections/licenses", "views/user/license_table_item", "cookiesjs"],
    function ($, _, Backbone, Templates, Events, User, Context, License, Licenses, LicenseTableItem, Cookies) {
      var LicenseView = Backbone.View.extend({
        template: Templates["user/license"],
        tableTemplate: Templates["user/license_table"],
        subviews: {},
        initialize: function () {

          // initialize models
          this.user = new User();

          _.bindAll(this, "success", "error", "setData", "addLicense",
              "checkEmptyState");
        },
        events: {
          "submit form": "saveForm"
        },
        saveForm: function (e) {
          e.preventDefault();

          var data = Context.serializeObj($(e.currentTarget));

          if (data.license && data.license.trim().length > 0) {

            this.$("form .alert-success").hide();
            this.$("p.error").remove();
            this.$(".loading-icon-wrapper").show();

            var license = new License();
            license.save(data, {
              success: this.success,
              error: this.error
            });

          }

        },
        success: function (model, data) {
          this.$(".loading-icon-wrapper").hide();
          this.$("form .alert-success").fadeIn();
          this.$("form textarea[name=license]").val("");

          // add model to collection
          this.licenses.add(model);
        },
        error: function (model, xhr) {
          var res = {};
          this.$(".loading-icon-wrapper").hide();

          try {
            res = $.parseJSON(xhr.responseText);
          } catch (e) {
            if (xhr.status === 401) {
              res = {error: xhr.responseText};
            } else {
              res = {error: "Unexpected response, please try again later or contact support."};
            }
          }

          var $error = $("<p>").addClass("error help-block").text(res.error ||
              "Unknown error, please try again later or contact support.");

          if (this.$("fieldset").is(":visible")) {
            $error.insertAfter(this.$("textarea[name=license]"));
          } else {
            $error.insertAfter(this.$("h2"));
          }
        },
        render: function () {
          // render template
          this.$el.html(this.template());

          // loading indicator
          var $loading = this.$(".loading-icon-wrapper");
          $loading.show();
          this.$("fieldset").hide();

          this.licenses = new Licenses();
          this.licenses.fetch({
            success: this.setData,
            error: this.error
          });
          this.licenses.on("add", this.addLicense);
          this.licenses.on("remove", this.checkEmptyState);

          return this;
        },
        checkEmptyState: function () {
          if (this.licenses.length === 0) {
            this.$("table").hide();
            License.clearLicense();
          } else {
            this.$("table").show();
            License.setLicense(this.licenses.last());
          }
        },
        addLicense: function (model) {
          var view = new LicenseTableItem(model);
          this.$("table>tbody").append(view.render().el);

          this.checkEmptyState();

          License.setLicense(model);
        },
        setData: function () {

          // loading indicator
          this.$(".loading-icon-wrapper").hide();
          this.$("fieldset").show();

          this.checkEmptyState();
        }
      });
      return LicenseView;
    });
